@@ -1,8 +1,18 @@
+# Copyright(c) 2014, The LIMIX developers (Christoph Lippert, Paolo Francesco Casale, Oliver Stegle)
+# All rights reserved.
+#
+# LIMIX is provided under a 2-clause BSD license.
+# See license.txt for the complete license.
+
 import scipy as SP
-import data_util as du
+import limix.modules.data_util as du
 import pandas as pd
 
 class genotype_reader_h5py():
+    """
+    a genotype reader using the HDF5 interface.
+    """
+
     def __init__(self,file_name):
         self.file_name = file_name
         self.load()
@@ -43,31 +53,27 @@ class genotype_reader_h5py():
         self.position=pd.DataFrame(data=position,index=self.geno_ID)        
 
         
-    def getGenotypes(self,sample_idx=None,idx_start=None,idx_end=None,pos_start=None,pos_end=None,chrom=None,center=True,unit=True,pos_cum_start=None,pos_cum_end=None,impute_missing=False,snp_idx=None):
+    def getGenotypes(self,sample_idx=None,idx_start=None,idx_end=None,chrom=None,pos_start=None,pos_end=None,center=True,unit=True,impute_missing=False,snp_idx=None,windowsize=0):
         """load genotypes. 
         Optionally the indices for loading subgroups the genotypes for all people
-        can be given in one out of three ways: 
-        - 0-based indexing (idx_start-idx_end)
-        - position (pos_start-pos_end on chrom)
-        - cumulative position (pos_cum_start-pos_cum_end)
-        If all these are None (default), then all genotypes are returned
+        based on position of cumulative position. 
+        Positions can be given as (pos_start-pos_end on chrom)
+        If both of these are None (default), then all genotypes are returned
 
         Args:
             idx_start:         genotype index based selection (start index)
             idx_end:         genotype index based selection (end index)
-            pos_start:       position based selection (start position)
-            pos_end:       position based selection (end position)
-            chrom:      position based selection (chromosome)
-            pos_cum_start:   cumulative position based selection (start position)
-            pos_cum_end:   cumulative position based selection (end position)
+            chrom:          chromosome based selection (return whole chromosome)
+            pos_start:       position based selection (start position) tuple of chrom, position
+            pos_end:       position based selection (end position) tuple of chrom, position
             impute_missing: Boolean indicator variable if missing values should be imputed
         
         Returns:
             X:          scipy.array of genotype values
         """
         #position based matching?
-        if (idx_start is None) and (idx_end is None) and ((pos_start is not None) & (pos_end is not None) & (chrom is not None)) or ((pos_cum_start is not None) & (pos_cum_end is not None)):
-            idx_start,idx_end=self.getGenoIndex(pos_start=pos_start,pos_end=pos_end,chrom=chrom,pos_cum_start=pos_cum_start,pos_cum_end=pose_cum1)
+        if (idx_start is None) and (idx_end is None) and ((pos_start is not None) & (pos_end is not None)):
+            idx_start,idx_end=self.getGenoIndex(chrom=chrom,pos_start=pos_start,pos_end=pos_end,windowsize=windowsize)
         #index based matching?
         if (idx_start is not None) & (idx_end is not None):
             X = self.geno_matrix[:,idx_start:idx_end]
@@ -81,35 +87,31 @@ class genotype_reader_h5py():
             X = du.imputeMissing(X,center=center,unit=unit)
         return X
 
-    def getGenoIndex(self,pos_start=None,pos_end=None,chrom=None,pos_cum_start=None,pos_cum_end=None):
+    def getGenoIndex(self,pos_start=None,pos_end=None,windowsize=0):
         """computes 0-based genotype index from position of cumulative position. 
-        Positions can be given in one out of two ways: 
-        - position (pos_start-pos_end on chrom)
-        - cumulative position (pos_cum_start-pos_cum_end)
-        If all these are None (default), then all genotypes are returned
+        Positions can be given as (pos_start-pos_end on chrom)
+        If both of these are None (default), then all genotypes are returned
 
         Args:
-            pos_start:       position based selection (start position)
-            pos_end:       position based selection (end position)
-            chrom:      position based selection (chromosome)
-            pos_cum_start:   cumulative position based selection (start position)
-            pos_cum_end:   cumulative position based selection (end position)
+            chrom:          chromosome based selection (return whole chromosome)
+            pos_start:      position based selection (start position) tuple of chrom, position
+            pos_end:        position based selection (end position) tuple of chrom, position
+            windowsize:     additionally include a flanking window around the selected positions (default 0)
         
         Returns:
             idx_start:         genotype index based selection (start index)
             idx_end:         genotype index based selection (end index)
         """
-        if (pos_start is not None) & (pos_end is not None) & (chrom is not None):
-            I = self.geno_chrom==chrom
-            I = I & (self.geno_pos>=p0) & (self.geno_pos<p1)
+        if (pos_start is not None) & (pos_end is not None):
+            assert pos_start[0]==pos_end[0],   "getGenoIndex only supports selection on a single chromosome"
+            I = self.position["chrom"]==pos_start[0]
+            I = I & (self.postion["pos"]>=(pos_start[1]-windowsize)) & (self.position["pos"]<(pos_end[1]+windowsize))
             I = SP.nonzero(I)[0]
             idx_start = I.min()
             idx_end = I.max()
-        elif (pos_cum_start is not None) & (pos_cum_end is not None):
-            I = (self.geno_pos_cum>=pos_cum_start) & (self.geno_pos_cum<pos_cum_end)
-            I = SP.nonzero(I)[0]
-            if I.size==0:
-                return None
+        elif (chrom is not None):
+            I = self.position["chrom"]==chrom
+            
             idx_start = I.min()
             idx_end = I.max()
         else:
@@ -117,7 +119,7 @@ class genotype_reader_h5py():
             idx_end=None
         return idx_start,idx_end
 
-    def getCovariance(self,sample_idx=None,normalize=False,idx_start=None,idx_end=None,pos_start=None,pos_end=None,chrom=None,center=True,unit=True,pos_cum_start=None,pos_cum_end=None,blocksize=None,X=None,snp_idx=None,**kw_args):
+    def getCovariance(self,sample_idx=None,normalize=True,idx_start=None,idx_end=None,pos_start=None,pos_end=None,chrom=None,center=True,unit=True,blocksize=None,X=None,snp_idx=None,windowsize=0,**kw_args):
         """calculate the empirical genotype covariance in a region"""
         if X is not None:
             if X.dtype!=SP.float64:
@@ -125,8 +127,8 @@ class genotype_reader_h5py():
             K=X.dot(X.T)
             Nsnp=X.shape[1]
         else:
-            if (idx_start is None) and (idx_end is None) and ((pos_start is not None) & (pos_end is not None) & (chrom is not None)) or ((pos_cum_start is not None) & (pos_cum_end is not None)):
-                idx_start,idx_end=self.getGenoIndex(pos_start=pos_start,pos_end=pos_end,chrom=chrom,pos_cum_start=pos_cum_start,pos_cum_end=pose_cum1)
+            if (idx_start is None) and (idx_end is None) and ((pos_start is not None) & (pos_end is not None) ) and (chrom is not None):
+                idx_start,idx_end=self.getGenoIndex(pos_start=pos_start,pos_end=pos_end,chrom=chrom,pos_cum_start=pos_cum_start,windowsize=windowsize)
 
             [N,M]=self.geno_matrix.shape
             if blocksize is None:
@@ -150,8 +152,8 @@ class genotype_reader_h5py():
                 nread+=thisblock
         if normalize:
             K/=(K.diagonal().mean())
-        else:#divide by number of SNPs in K
-            K/=Nsnp
+        #else:#divide by number of SNPs in K
+        #    K/=Nsnp
         return K
 
     def getGenoID(self,snp_idx=None,idx_start=None,idx_end=None,pos_start=None,pos_end=None,chrom=None,pos_cum_start=None,pos_cum_end=None):
@@ -303,7 +305,7 @@ class genotype_reader_tables():
             X = du.imputeMissing(X,center=center,unit=unit)
         return X
 
-    def getGenoIndex(self,pos_start=None,pos_end=None,chrom=None,pos_cum_start=None,pos_cum_end=None):
+    def getGenoIndex(self,pos_start=None,pos_end=None,chrom=None,windowsize=0):
         """computes 0-based genotype index from position of cumulative position. 
         Positions can be given in one out of two ways: 
         - position (pos_start-pos_end on chrom)
@@ -314,21 +316,20 @@ class genotype_reader_tables():
             pos_start:       position based selection (start position)
             pos_end:       position based selection (end position)
             chrom:      position based selection (chromosome)
-            pos_cum_start:   cumulative position based selection (start position)
-            pos_cum_end:   cumulative position based selection (end position)
         
         Returns:
             idx_start:         genotype index based selection (start index)
             idx_end:         genotype index based selection (end index)
         """
-        if (pos_start is not None) & (pos_end is not None) & (chrom is not None):
-            I = self.geno_chrom==chrom
-            I = I & (self.geno_pos>=p0) & (self.geno_pos<p1)
+        if (pos_start is not None) & (pos_end is not None):
+            assert pos_start[0] == pos_end[0], "chromosomes don't match between start and end position"
+            I = self.position["chrom"]==pos_start[0]
+            I = I & (self.position["pos"]>=(pos_start[1]-windowsize)) & (self.position["pos"]<(pos_end[1]+windowsize))
             I = SP.nonzero(I)[0]
             idx_start = I.min()
             idx_end = I.max()
-        elif (pos_cum_start is not None) & (pos_cum_end is not None):
-            I = (self.geno_pos_cum>=pos_cum_start) & (self.geno_pos_cum<pos_cum_end)
+        elif chrom is not None:
+            I = self.position["chrom"]==chrom
             I = SP.nonzero(I)[0]
             if I.size==0:
                 return None
@@ -339,7 +340,7 @@ class genotype_reader_tables():
             idx_end=None
         return idx_start,idx_end
 
-    def getCovariance(self,sample_idx=None,normalize=False,idx_start=None,idx_end=None,pos_start=None,pos_end=None,chrom=None,center=True,unit=True,pos_cum_start=None,pos_cum_end=None,blocksize=5000,X=None,snp_idx=None,**kw_args):
+    def getCovariance(self,sample_idx=None,normalize=False,idx_start=None,idx_end=None,pos_start=None,pos_end=None,chrom=None,windowsize=0,center=True,unit=True,blocksize=5000,X=None,snp_idx=None,**kw_args):
         """calculate the empirical genotype covariance in a region"""
         if X is not None:
             if X.dtype!=SP.float64:
@@ -347,8 +348,8 @@ class genotype_reader_tables():
             K=X.dot(X.T)
             Nsnp=X.shape[1]
         else:
-            if (idx_start is None) and (idx_end is None) and ((pos_start is not None) & (pos_end is not None) & (chrom is not None)) or ((pos_cum_start is not None) & (pos_cum_end is not None)):
-                idx_start,idx_end=self.getGenoIndex(pos_start=pos_start,pos_end=pos_end,chrom=chrom,pos_cum_start=pos_cum_start,pos_cum_end=pose_cum1)
+            if (idx_start is None) and (idx_end is None) and ((pos_start is not None) & (pos_end is not None) ) or (chrom is not None):
+                idx_start,idx_end=self.getGenoIndex(pos_start=pos_start,pos_end=pos_end,chrom=chrom,windowsize=windowsize)
 
             [N,M]=self.geno_matrix.shape
             if blocksize is None:
@@ -372,11 +373,11 @@ class genotype_reader_tables():
                 nread+=thisblock
         if normalize:
             K/=(K.diagonal().mean())
-        else:#divide by number of SNPs in K
-            K/=Nsnp
+        #else:#divide by number of SNPs in K
+        #    K/=Nsnp
         return K
 
-    def getGenoID(self,snp_idx=None,idx_start=None,idx_end=None,pos_start=None,pos_end=None,chrom=None,pos_cum_start=None,pos_cum_end=None):
+    def getGenoID(self,snp_idx=None,idx_start=None,idx_end=None,pos_start=None,pos_end=None,chrom=None,windowsize=0):
         """get genotype IDs. 
         Optionally the indices for loading subgroups the genotype IDs for all people
         can be given in one out of three ways: 
@@ -391,15 +392,13 @@ class genotype_reader_tables():
             pos_start:       position based selection (start position)
             pos_end:       position based selection (end position)
             chrom:      position based selection (chromosome)
-            pos_cum_start:   cumulative position based selection (start position)
-            pos_cum_end:   cumulative position based selection (end position)
            
         Returns:
             ID:         scipy.array of genotype IDs (e.g. rs IDs)
         """
         #position based matching?
-        if (idx_start is None) and (idx_end is None) and ((pos_start is not None) & (pos_end is not None) & (chrom is not None)) or ((pos_cum_start is not None) & (pos_cum_end is not None)):
-            idx_start,idx_end=self.getGenoIndex(pos_start=pos_start,pos_end=pos_end,chrom=chrom,pos_cum_start=pos_cum_start,pos_cum_end=pose_cum1)
+        if (idx_start is None) and (idx_end is None) and ((pos_start is not None) & (pos_end is not None) ) or (chrom is not None):
+            idx_start,idx_end=self.getGenoIndex(pos_start=pos_start,pos_end=pos_end,chrom=chrom,windowsize=windowsize)
         if "genotype_id" in self.geno:
             if (idx_start is not None) & (idx_end is not None):
                 return self.geno.genotype_id[idx_start:idx_end]
@@ -415,7 +414,7 @@ class genotype_reader_tables():
             else:
                 return SP.arange(self.geno_matrix.shape[1])
 
-    def getPos(selfidx_start=None,idx_end=None,pos_start=None,pos_end=None,chrom=None,pos_cum_start=None,pos_cum_end=None):
+    def getPos(selfidx_start=None,idx_end=None,pos_start=None,pos_end=None,chrom=None,windowsize=0):
         """
         get the positions of the genotypes
 
@@ -424,8 +423,8 @@ class genotype_reader_tables():
             position
             cumulative_position
         """
-        if (idx_start is None) and (idx_end is None) and ((pos_start is not None) & (pos_end is not None) & (chrom is not None)) or ((pos_cum_start is not None) & (pos_cum_end is not None)):
-            idx_start,idx_end=self.getGenoIndex(pos_start=pos_start,pos_end=pos_end,chrom=chrom,pos_cum_start=pos_cum_start,pos_cum_end=pose_cum1)
+        if (idx_start is None) and (idx_end is None) and ((pos_start is not None) & (pos_end is not None) ) or (chrom is not None):
+            idx_start,idx_end=self.getGenoIndex(pos_start=pos_start,pos_end=pos_end,chrom=chrom,windowsize=windowsize)
         if (idx_start is not None) & (idx_end is not None):
             return self.geno_chrom[idx_start:idx_end],self.geno_pos[idx_start:idx_end]
         elif snp_idx is not None:
@@ -434,14 +433,6 @@ class genotype_reader_tables():
             return self.geno_chrom,self.geno_pos
 
 
-    def getIcis_geno(self,geneID,cis_window=50E3):
-        """ if eqtl==True it returns a bool vec for cis """
-        assert self.eqtl == True, 'Only for eqtl data'
-        index = self.geneID==geneID
-        [_chrom,_gene_start,_gene_start] = self.gene_pos[index][0,:]
-        Icis = (self.geno_chrom==_chrom)*(self.geno_pos>=_gene_start-cis_window)*(self.geno_pos<=_gene_start+cis_window)
-        return Icis
-
 """
     def to_table(self, filename):
         file = tables.open 
